import pandas as pd
import plotly.graph_objects as go
from dash import Dash, html, dcc
from dash.dependencies import Input, Output
from drone_troopers.preprocessing import preprocess, create_time_series_features
import pickle
import os
import numpy as np
import time
from drone_troopers import COLUMNS_CONSIDERED
import shap

def get_resource_path(filename):
    """Get the absolute path of a resource file."""
    return os.path.join(os.path.dirname(__file__), filename)


def add_lags(df, col_name, lags, fill_na=False):
    df = df.copy()
    for lag in lags:
        df[f"{col_name}_lag_{lag}"] = df[col_name].shift(lag)
        if fill_na:
            df[f"{col_name}_lag_{lag}"] = df[f"{col_name}_lag_{lag}"].fillna(0)
    return df


def mark_state_ON(
    df, window_size=10, rolling_current_thresh=20, rolling_pow_thresh=30000
):
    df["State_ON"] = 0
    rolling_battery_current_sum = (
        df["Battery Current (A)"].rolling(window=window_size).sum()
    )
    rolling_power_generated_sum = (
        df["Power Generated (W)"].rolling(window=window_size).sum()
    )

    rc_exceeding_thresh = rolling_battery_current_sum > rolling_current_thresh
    pow_exceeding_thresh = rolling_power_generated_sum > rolling_pow_thresh

    index_exceeding = df.index[(rc_exceeding_thresh & pow_exceeding_thresh) == True]
    if len(index_exceeding) > 0:
        last_idx = df.index[-1]
        df.loc[index_exceeding[0] :, "State_ON"] = 1
        df.loc[last_idx:, "State_ON"] = 0

    return df


def find_consecutive_indices(arr):
    arr = np.asarray(arr)
    diff = np.diff(arr)
    starts = np.where(diff == 1)[0] + 1
    ends = np.where(diff == -1)[0]

    if arr[0] == 1:
        starts = np.insert(starts, 0, 0)
    if arr[-1] == 1:
        ends = np.append(ends, len(arr) - 1)

    result = []
    for start, end in zip(starts, ends):
        if start == end:
            start -= 1 if start > 0 else 0
        result.append((start, end))

    return result

def model_predict(X):
    return model.predict_proba(X)

with open(get_resource_path(os.path.join("..", "models", "voting_classifier_samples_model.pkl")), "rb") as file:
    model = pickle.load(file)

with open(get_resource_path(os.path.join("..", "models", "std_scaler_samples_model.pkl")),"rb") as file:
    std_scaler = pickle.load(file)

# Variables to store the simulation parameters
START_IDX = 100  # Define the starting index for simulation
MAX_WINDOW_SIZE = 99  # Maximum number of rows in current_df
i = START_IDX  # Start from START_IDX

samples_label = []
data_model_columns = None

# Load initial data from Excel file
excel_file_path = r"C:\Python Programs\Tech-Troopers\Dhruv\Drone\logs\selected\-1\50 throttle not enough power(annotated).xlsx"
csv_file_path = r'C:\Python Programs\Tech-Troopers\Dhruv\Drone\logs\selected\-1\100% throttle no power.csv'
# df = preprocess(pd.read_excel(excel_file_path))
df = preprocess(pd.read_csv(csv_file_path))
df = mark_state_ON(df)
data_model = create_time_series_features(df.drop(columns='State_ON'))
data_model = data_model.reset_index(drop=True)

for col in COLUMNS_CONSIDERED:
    data_model = add_lags(data_model, col, range(1, 10), fill_na=True)
data_model_columns = data_model.columns
data_model = std_scaler.transform(data_model)
df['alert'] = model.estimators_[-1].predict(data_model)

# Calculate SHAP values
explainer = shap.Explainer(model.estimators_[-1], data_model)
shap_values = explainer(data_model)
del data_model
# del data_model

# Create a Dash web application
app = Dash(__name__)

# Define the layout of the application with a 2x2 grid
app.layout = html.Div(
    [
        html.Div(
            [
                dcc.Graph(id=f"plot-{column}", style={"height": "400px"})
                for column in COLUMNS_CONSIDERED[:4]
            ],  # First row
            style={
                "display": "grid",
                "gridTemplateColumns": "repeat(2, 1fr)",
                "gap": "10px",
            },
        ),
        html.Div(
            [
                dcc.Graph(id=f"plot-{column}", style={"height": "400px"})
                for column in COLUMNS_CONSIDERED[4:8]
            ],  # Second row
            style={
                "display": "grid",
                "gridTemplateColumns": "repeat(2, 1fr)",
                "gap": "10px",
            },
        ),
        html.Div(
            [
                dcc.Graph(id=f"plot-{column}", style={"height": "400px"})
                for column in COLUMNS_CONSIDERED[8:12]
            ],  # Third row
            style={
                "display": "grid",
                "gridTemplateColumns": "repeat(2, 1fr)",
                "gap": "10px",
            },
        ),
        html.Div(
            [
                dcc.Graph(id=f"plot-{column}", style={"height": "400px"})
                for column in COLUMNS_CONSIDERED[12:]
            ],  # Fourth row
            style={
                "display": "grid",
                "gridTemplateColumns": "repeat(2, 1fr)",
                "gap": "10px",
            },
        ),
        dcc.Interval(
            id="interval-component", interval=2000, n_intervals=0  # in milliseconds
        ),
    ]
)


# Define callback to update the plots in real-time
@app.callback(
    [Output(f"plot-{column}", "figure") for column in COLUMNS_CONSIDERED],
    [Input("interval-component", "n_intervals")],
)
def update_graph(n_intervals):
    global i, std_scaler, model, samples_label
    i += 1

    # Ensure we don't exceed the bounds of df
    if i >= len(df):
        i = len(df) - 1

    # Create current_data based on the starting index and max window size
    start_index = max(0, i - MAX_WINDOW_SIZE + 1)  # Calculate start index
    current_data = df.iloc[start_index : i + 1]  # Slicing current_data based on indices
    if len(samples_label) == 0:
        samples_label = [0] * (len(current_data) - 1)
    else:
        samples_label = samples_label[-(len(current_data) - 1) :]

    assert len(current_data) - 1 == len(
        samples_label
    ), f"Length mismatch: current_data length is {len(current_data) - 1}, samples_label length is {len(samples_label)}"
    change_indices = current_data.index[
        current_data["State_ON"] != current_data["State_ON"].shift()
    ]

    # Separate into two lists
    indices_0_to_1 = change_indices[
        current_data.loc[change_indices, "State_ON"] == 1
    ].tolist()
    indices_1_to_0 = change_indices[
        current_data.loc[change_indices, "State_ON"] == 0
    ].tolist()

    inFlight = current_data.iloc[-1]["State_ON"] == 1
    if True:
        label = int(current_data.iloc[-1]['alert'])
    else:
        label = 0

    samples_label.append(label)

    # Create a list to store figures for each column
    figures = []

    # Loop through the columns and create a trace for each
    for column in COLUMNS_CONSIDERED:
        if column in current_data.columns:  # Check if the column exists
            fig = go.Figure()

            fig.add_trace(
                go.Scatter(
                    x=current_data.index.time,  # Use the time part of the index
                    y=current_data[column],
                    mode="lines",
                    name=column,
                )
            )

            # Update layout for individual plot
            fig.update_layout(
                title=column,
                xaxis_title="Time",
                yaxis_title="Values",
                legend_title="Metrics",
                template="plotly",
                xaxis=dict(
                    tickangle=-45, tickfont=dict(size=8)
                ),  # Smaller x-axis tick labels
                yaxis=dict(tickfont=dict(size=8)),  # Smaller y-axis tick labels
                height=400,  # Set the height of the plot
            )

            for idx in indices_0_to_1:
                fig.add_vline(
                    idx.time(),
                    line_width=2,
                    line_dash="dash",
                    line_color="green",
                    layer="below",
                    opacity=0.5,
                )
            for idx in indices_1_to_0:
                if current_data.index[0] != idx:
                    fig.add_vline(
                        idx.time(),
                        line_width=2,
                        line_dash="dash",
                        line_color="red",
                        layer="below",
                        opacity=0.5,
                    )

            if len(samples_label) != len(current_data):
                samples_label = samples_label[-len(current_data) :]
            assert len(current_data) == len(
                samples_label
            ), f"Length mismatch2: current_data length is {len(current_data)}, samples_label length is {len(samples_label)}"
            consecutive_marked_indices = find_consecutive_indices(samples_label)
            for start_idx, end_idx in consecutive_marked_indices:
                if end_idx < len(current_data):
                    highlight_x0 = current_data.index[start_idx].time()
                    highlight_x1 = current_data.index[end_idx].time()
                    highlight_y_min = current_data[column].min()
                    highlight_y_max = current_data[column].max()
                    fig.add_shape(
                        type="rect",
                        xref="x",
                        yref="y",
                        x0=highlight_x0,
                        y0=highlight_y_min,
                        x1=highlight_x1,
                        y1=highlight_y_max,
                        fillcolor="yellow",
                        opacity=0.5,
                        line_width=0,
                        layer="below",
                    )

            # Add annotation for the label
            fig.add_annotation(
                text=f"Label: {'Normal Operation' if label == 0 else 'Non Normal Operation'}",
                xref="paper",
                yref="paper",
                x=0.5,
                y=1.1,
                showarrow=False,
                font=dict(size=16, color="black"),
                align="center",
                bgcolor="white",    
                opacity=0.8,
            )
            
            current_sample_index = i
            shap_val = shap_values[current_sample_index]    
            shap_df = pd.DataFrame(shap_val.values.reshape(1, -1), columns=data_model_columns)
                
            # Identify the most important feature
            most_important_feature = shap_df.abs().mean().idxmax()

            # Remove lag suffix if it exists
            if '_lag_' in most_important_feature:
                most_important_feature = most_important_feature.split('_lag_')[0]
                
            if most_important_feature in ['hour', 'second', 'minute']:
                # If it's one of the time features, find the next best feature
                second_most_important_feature = shap_df.abs().mean().nlargest(2).idxmin()
                feature_to_display = second_most_important_feature
            else:
                feature_to_display = most_important_feature
                
            # Add the feature importance annotation
            fig.add_annotation(
                text=f"Most Important Feature: {feature_to_display}",
                xref="paper",
                yref="paper",
                x=0.5,
                y=-0.15,
                showarrow=False,
                font=dict(size=12, color="black"),
                align="center",
                bgcolor="white",
                opacity=0.8,
            )

            figures.append(fig)

    return figures



# Run the application
if __name__ == "__main__":

    # ===========RUN SERVER==============
    app.run_server(debug=True, host="0.0.0.0")



import pickle
import os
import numpy as np
from drone_troopers.preprocessing import preprocess, create_time_series_features, add_lags
from drone_troopers import COLUMNS_CONSIDERED
import pandas as pd

def get_resource_path(filename):
    """Get the absolute path of a resource file."""
    return os.path.join(os.path.dirname(__file__), filename)

def get_filename_from_path(file_path, without_extension=False):
    """
    Extracts the filename from a given file path.
    
    Args:
    file_path (str): The full path to the file.
    
    Returns:
    str: The filename extracted from the path.
    """
    file_name = os.path.basename(file_path)
    if without_extension :
        file_name, _ = os.path.splitext(file_name)
    return file_name


def read_files(lst) :
    final_df = None
    for idx, x in enumerate(lst) :
        if x.lower().endswith('.csv') :
            df = preprocess(pd.read_csv(x))
        else :
            df = preprocess(pd.read_excel(x))
        df['flightId'] = idx
        
        df = create_time_series_features(df) 
        df = df.reset_index(drop=True)

        for col in COLUMNS_CONSIDERED:
            df = add_lags(df, col, range(1, 10), fill_na=True)

        if final_df is None :
            final_df = df.copy()
        else :
            final_df = pd.concat([final_df, df], axis=0)


    return final_df

with open(get_resource_path(os.path.join("std_scaler_samples_model.pkl")),"rb") as file:
    std_scaler = pickle.load(file)

def scale_n_process(df) :
    return std_scaler.transform(df)

def inverse_transform(data_scaled, columns):
    df_original = std_scaler.inverse_transform(data_scaled)
    return pd.DataFrame(df_original, columns=columns)

def predict(lst) :
    final_df = read_files(lst)
    flight_Ids = final_df['flightId']
    final_df = final_df.drop(columns='flightId')
    columns = final_df.columns
    scaled_data = scale_n_process(final_df)
    
    scaled_data = pd.DataFrame(scaled_data, columns=columns)
    # scaled_data['flightId'] = flight_Ids

    with open(get_resource_path(os.path.join("voting_classifier_samples_model.pkl")), "rb") as file:
        model = pickle.load(file)
    y_pred = model.predict(scaled_data.values)
    scaled_data['alert'] = y_pred

     # Group by flightId
    grouped_predictions = scaled_data.groupby(flight_Ids)

    # Create a dictionary to hold dataframes for each flightId
    prediction_dfs = {flight_id: group for flight_id, group in grouped_predictions}
    return [x['alert'].values for x in list(prediction_dfs.values())]

def predict_n_save(lst, save_paths=None) :
    if save_paths is not None :
        assert len(lst) == len(save_paths), 'Saved paths list must be same length as input list'
    final_df = read_files(lst)
    flight_Ids = final_df['flightId']
    final_df = final_df.drop(columns='flightId')
    columns = final_df.columns
    scaled_data = scale_n_process(final_df)
    scaled_data = pd.DataFrame(scaled_data, columns=columns)

    with open(get_resource_path(os.path.join("..", "models", "voting_classifier_samples_model.pkl")), "rb") as file:
        model = pickle.load(file)
    y_pred = model.predict(scaled_data.values)
    scaled_data['alert'] = y_pred

     # Group by flightId
    grouped_predictions = scaled_data.groupby(flight_Ids)

    # Create a dictionary to hold dataframes for each flightId
    for idx, (flight_id, group) in enumerate(grouped_predictions) :
        if save_paths is not None :
            path_to_save = save_paths[idx]
        else :
            file_name = get_filename_from_path(lst[idx], without_extension=True)
            path_to_save = f'{file_name}_annotated.csv'

        group_transformed = inverse_transform(group.drop(columns='alert').values, columns)
        group_transformed = group_transformed[COLUMNS_CONSIDERED]
        group = pd.concat((group_transformed, group['alert']), axis=1)
        group.to_csv(path_to_save, index=False)
        print(f'Saved {path_to_save}')


if __name__ == '__main__' :
    path1 = r'C:\Python Programs\Tech-Troopers\Dhruv\Drone\logs\selected\-1\50 throttle not enough power(annotated).xlsx'
    path3 = r'C:\Python Programs\Tech-Troopers\Dhruv\Drone\logs\selected\-1\NoEngineCranking\B3P 041\LOG00000\LOG.csv'
    path2 = r'C:\Python Programs\Tech-Troopers\Dhruv\Drone\logs\selected\1\1\LOG00000\LOG.csv'
    path4 = r'C:\Python Programs\Tech-Troopers\Dhruv\Drone\logs\selected\1\1\LOG00001\LOG.csv'
    
    # print(predict([path2]))
    predict_n_save([path2], ['path2_annotated.csv'])








import numpy as np
import re
import pandas as pd


column_patterns = {
    'Time (s)': r'Time.*s',
    'Motor Speed (RPM)': r'Motor.*RPM',
    'Engine Speed (RPM)': r'Engine.*Speed.*RPM',
    'Throttle (%)': r'Throttle.*%',
    'Intake Temperature (C)': r'\s*Intake\s*Temp(?:erature)?\s*\(\s*C\s*\)',
    'Engine Coolant Temperature 1 (C)': r'\s*Engine\s*Coolant\s*(?:Temperature|Temp)\s*1?\s*\(\s*C\s*\)',
    'Engine Coolant Temperature 2 (C)': r'\s*Engine\s*Coolant\s*(?:Temperature|Temp)\s*2?\s*\(\s*C\s*\)',
    'Barometric Pressure (kpa)': r'Barometric.*Pressure.*kpa',
    'Fuel Trim': r'Fuel.*Trim',
    'Fuel Consumption (g/min)': r'Fuel.*Consumption.*g.*min',
    'Fuel Consumed (g)': r'Fuel.*Consumed.*g',
    # 'Expected Max Power (W)': r'Expected.*Max.*Power.*W',
    'Bus Voltage (V)': r'Bus.*Voltage.*V',
    'Battery Current (A)': r'Battery.*Current.*A',
    'Power Generated (W)': r'Power.*Generated.*W',
    'Inverter Temperature (C)': r'\s*Inverter\s*(?:Temperature|MAX)\s*\(\s*C\s*\)',
    'Target Fuel Pressure (bar)': r'Target.*Fuel.*Pressure.*bar',
    'Fuel Pressure (bar)': r'Fuel.*Pressure.*bar',
    'Fuel Pump Speed (RPM)': r'Fuel.*Pump.*Speed.*RPM',
    'Cooling Pump Speed (RPM)': r'Cooling.*Pump.*Speed.*RPM',
}

def add_lags(df, col_name, lags, fill_na=False):
    df = df.copy()
    for lag in lags:
        df[f"{col_name}_lag_{lag}"] = df[col_name].shift(lag)
        if fill_na:
            df[f"{col_name}_lag_{lag}"] = df[f"{col_name}_lag_{lag}"].fillna(0)
    return df


def mark_state_ON(
    df, window_size=10, rolling_current_thresh=20, rolling_pow_thresh=30000
):
    df["State_ON"] = 0
    rolling_battery_current_sum = (
        df["Battery Current (A)"].rolling(window=window_size).sum()
    )
    rolling_power_generated_sum = (
        df["Power Generated (W)"].rolling(window=window_size).sum()
    )

    rc_exceeding_thresh = rolling_battery_current_sum > rolling_current_thresh
    pow_exceeding_thresh = rolling_power_generated_sum > rolling_pow_thresh

    index_exceeding = df.index[(rc_exceeding_thresh & pow_exceeding_thresh) == True]
    if len(index_exceeding) > 0:
        last_idx = df.index[-1]
        df.loc[index_exceeding[0] :, "State_ON"] = 1
        df.loc[last_idx:, "State_ON"] = 0

    return df


def find_consecutive_indices(arr):
    arr = np.asarray(arr)
    diff = np.diff(arr)
    starts = np.where(diff == 1)[0] + 1
    ends = np.where(diff == -1)[0]

    if arr[0] == 1:
        starts = np.insert(starts, 0, 0)
    if arr[-1] == 1:
        ends = np.append(ends, len(arr) - 1)

    result = []
    for start, end in zip(starts, ends):
        if start == end:
            start -= 1 if start > 0 else 0
        result.append((start, end))

    return result

def standardize_columns(df, column_patterns, retain_all_columns=False):
    standardized_columns = {}
    for standard_col, pattern in column_patterns.items():
        for col in df.columns:
            if re.match(pattern, col, re.IGNORECASE):
                standardized_columns[col] = standard_col
                break
    df = df.rename(columns=standardized_columns)
    
    return df if retain_all_columns else df[list(column_patterns.keys())]

def preprocess(df, filter_throttle_values=None, retain_all_columns=False) :
    df = df.copy()
    df = standardize_columns(df, column_patterns, retain_all_columns)
    if filter_throttle_values is not None and df['Throttle (%)'].max() < filter_throttle_values :
        return None
    if retain_all_columns :
        time_col_original = df['Time (s)']
    df = df.dropna()
    reference_date = pd.to_datetime('1970-01-01')
    df['Time (s)'] = reference_date + pd.to_timedelta(df['Time (s)'], unit='s')
    df = df.set_index('Time (s)')
    if retain_all_columns :
        df = pd.concat([time_col_original, df], axis=1)
    return df

def create_time_series_features(df) :
    df = df.copy()
    df['hour'] = df.index.hour
    df['minute'] = df.index.minute
    df['second'] = df.index.second
    df['microsecond'] = df.index.microsecond
    return df


import sys
import subprocess
import os
import tempfile
import shutil


def get_resource_path(filename):
    """Get the absolute path of a resource file."""
    return os.path.join(os.path.dirname(__file__), filename)

def escape_latex_special_chars(filename):
    """Escape special characters in the filename for LaTeX."""
    special_chars = {
        '&': '\\&',
        '%': '\\%',
        '$': '\\$',
        '#': '\\#',
        '_': '\\_',
        '{': '\\{',
        '}': '\\}',
        '~': '\\textasciitilde',
        '^': '\\textasciicircum',
        '\\': '\\textbackslash'
    }
    for char, escape in special_chars.items():
        filename = filename.replace(char, escape)
    return filename

def update_latex_content(filename):
    # Path to the LaTeX file
    latex_file_path = get_resource_path('main.tex')
    
    # Escape special characters in the filename
    escaped_filename = escape_latex_special_chars(os.path.basename(filename))
    
    # Read the existing content of the LaTeX file
    with open(latex_file_path, 'r') as file:
        content = file.readlines()

    # Replace the line containing the filename
    for i, line in enumerate(content):
        if 'filename.csv' in line:
            content[i] = line.replace('filename.csv', escaped_filename)
            break
        
    return '\n'.join(content)

def compile_raw_latex(raw_latex_code, resource_paths=['img']):
    with tempfile.TemporaryDirectory() as tempdir:
        tex_file_path = os.path.join(tempdir, 'temp.tex')
        
        # Write the raw LaTeX code to a temporary file
        with open(tex_file_path, 'w') as tex_file:
            tex_file.write(raw_latex_code)
            
        for resource_dir in resource_paths:
            resource_dir_path = get_resource_path(resource_dir)
            for item in os.listdir(resource_dir_path):
                s = os.path.join(resource_dir_path, item)
                d = os.path.join(tempdir, item)
                if os.path.isdir(s):
                    shutil.copytree(s, d)
                else:
                    shutil.copy2(s, d)
            
        subprocess.run(['pdflatex', '-jobname=report', tex_file_path], check=True)
            
    # # Use subprocess to run pdflatex and pass the raw LaTeX code
    # process = subprocess.Popen(
    #     ['pdflatex', '-interaction=nonstopmode', '-halt-on-error', '-jobname=report', tex_file_path],
    #     stdin=subprocess.PIPE,
    #     stdout=subprocess.PIPE,
    #     stderr=subprocess.PIPE,
    #     text=True,
    #     cwd=get_resource_path(tempdir)  # Set the working directory
    # )
    
    # # Send the raw LaTeX code to pdflatex
    # stdout, stderr = process.communicate()
    
    # # Check for errors
    # if process.returncode != 0:
    #     print("Error in LaTeX compilation:")
    #     print(stderr)
    # else:
    #     print("LaTeX compilation successful!")
    #     print(stdout)

def compile_latex():
    latex_file_path = get_resource_path('main.tex')
    # Compile the LaTeX file to PDF
    subprocess.run(['pdflatex', '-jobname=report', latex_file_path], check=True)

def open_pdf():
    # Open the generated PDF file
    if os.name == 'nt':  # For Windows
        os.startfile('main.pdf')
    elif os.name == 'posix':  # For macOS and Linux
        subprocess.run(['open', 'report.pdf'])  # For macOS
        # subprocess.run(['xdg-open', 'report.pdf'])  # For Linux

if __name__ == "__main__":
    # Check if a filename is provided as a command-line argument
    if len(sys.argv) > 1:
        filename = sys.argv[1]
    else:
        path1 = r'C:\Python Programs\Tech-Troopers\Dhruv\Drone\logs\selected\-1\50 throttle not enough power(annotated).xlsx'
        filename = path1  # Default filename if none provided

    content = update_latex_content(filename)
    print(f'Updated filename in main.tex to: {filename}')

    compile_raw_latex(content)
    print('Compilation completed.')

    # open_pdf()
    print('Opened report.pdf.')



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from matplotlib import style
style.use('ggplot')
import seaborn as sns
import re
import os
import pickle
from glob import glob
from drone_troopers.preprocessing import preprocess, create_time_series_features

# Define column indices for PCA
pca_column_indices = list(range(0, 18))
pca_column_indices.remove(2) # Indices for the columns from 'Motor Speed (RPM)' to 'Cooling Pump Speed (RPM)'

# Column index to scale using MinMaxScaler
throttle_column_indices = [2]  # Index for 'Throttle (%)'

# Columns to keep unchanged (by indices)
unchanged_column_indices = list(range(18, 23))  # Indices for 'flightId', 'hour', 'minute', 'second', 'microsecond'

def get_resource_path(filename):
    """Get the absolute path of a resource file."""
    return os.path.join(os.path.dirname(__file__), filename)

def scale_n_process(time_series_data) :
    with open(get_resource_path('preprocessor.pkl'), 'rb') as file:
        preprocessor = pickle.load(file)
    
    with open(get_resource_path('data_column_names.pkl'), 'rb') as file:
        data_column_names = pickle.load(file)
        
    transformed_data = preprocessor.transform(time_series_data)
    # Convert the transformed data back to a DataFrame for better readability
    # The output column names after PCA will be 'PC1', 'PC2', ..., 'PCn' where n is the number of components
    pca_feature_names = [f'PC{i+1}' for i in range(transformed_data.shape[1] - len(unchanged_column_indices) - 1)]
    final_column_names = pca_feature_names + ['Throttle (%)'] + [data_column_names[i] for i in unchanged_column_indices]

    final_df = pd.DataFrame(transformed_data, columns=final_column_names)
    return final_df
    
def read_files(lst) :
    time_series_data = np.empty((0, 23))
    for idx, x in enumerate(lst) :
        if x.lower().endswith('.csv') :
            df = preprocess(pd.read_csv(x))
        else :
            df = preprocess(pd.read_excel(x))
            
        df['flightId'] = idx
        
        df = create_time_series_features(df)        
        time_series_data = np.concatenate((time_series_data, df.values), axis=0)
    return time_series_data

def predict(lst) :
    time_series_data = read_files(lst)
    final_df = scale_n_process(time_series_data)
    
    with open(get_resource_path('voting_classifier_model.pkl'), 'rb') as file:
        voting_clf = pickle.load(file)
        
    flightId_grouped = final_df.groupby('flightId').size()
    y_pred = voting_clf.predict(final_df.drop('flightId', axis=1))
    final_y = []
    start_idx = 0
    for idx, val in flightId_grouped.items() :
        
        dataset_pred = y_pred[start_idx: start_idx + val]
        mode = pd.Series(dataset_pred).value_counts().sort_values(ascending=False).index[0]
        final_y.append(mode)
        start_idx += val
    return final_y

if __name__ == '__main__' :
    path1 = r'C:\Python Programs\Tech-Troopers\Dhruv\Drone\logs\selected\-1\50 throttle not enough power(annotated).xlsx'
    path3 = r'C:\Python Programs\Tech-Troopers\Dhruv\Drone\logs\selected\-1\NoEngineCranking\B3P 041\LOG00000\LOG.csv'
    path2 = r'C:\Python Programs\Tech-Troopers\Dhruv\Drone\logs\selected\1\1\LOG00000\LOG.csv'
    path4 = r'C:\Python Programs\Tech-Troopers\Dhruv\Drone\logs\selected\1\1\LOG00001\LOG.csv'
    
    print(predict([path1, path2, path3, path4]))
    


COLUMNS_CONSIDERED = [
    'Motor Speed (RPM)',
    'Engine Speed (RPM)',
    'Throttle (%)',
    'Intake Temperature (C)',
    'Engine Coolant Temperature 1 (C)',
    'Engine Coolant Temperature 2 (C)',
    'Barometric Pressure (kpa)',
    'Fuel Trim',
    'Fuel Consumption (g/min)',
    'Fuel Consumed (g)',
    'Bus Voltage (V)',
    'Battery Current (A)',
    'Power Generated (W)',
    'Inverter Temperature (C)',
    'Target Fuel Pressure (bar)',
    'Fuel Pressure (bar)',
    'Fuel Pump Speed (RPM)',
    'Cooling Pump Speed (RPM)',
]



from setuptools import setup, find_packages

setup(
    name='drone_troopers',
    version='0.0.1',
    author='Tech Troopers',
    author_email='dhruvyadav2905@gmail.com',
    description='A module for drone data processing',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    # url='https://github.com/yourusername/drone_troopers',
    packages=find_packages(),
    install_requires=[
        # Add your dependencies here
        'numpy',
        'pandas',
        'matplotlib',
        'scikit-learn',
        'xgboost',
        # Example: 'numpy>=1.18.0',
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)
